import abc
import os
from typing import Any, Awaitable, Dict, Union

import jwt

from original_sdk.types.environment import Environment, get_environment
from original_sdk.types.exceptions import is_error_status_code, parse_and_raise_error
from original_sdk.types.original_response import OriginalResponse

DEVELOPMENT_BASE_URL = "https://api-dev.getoriginal.com"
PRODUCTION_BASE_URL = "https://api.getoriginal.com"
DEFAULT_API_VERSION = "v1"


class BaseOriginalClient(abc.ABC):
    def __init__(
        self, api_key: str, api_secret: str, timeout: float = 6.0, **options: Any
    ):
        self.api_key = api_key
        self.api_secret = api_secret
        self.timeout = timeout

        if os.getenv("ORIGINAL_TIMEOUT"):
            self.timeout = float(os.environ["ORIGINAL_TIMEOUT"])

        self.options = options
        self.base_url = PRODUCTION_BASE_URL
        self.env = None
        self.api_version = DEFAULT_API_VERSION

        if options.get("base_url"):
            self.base_url = options["base_url"]
        elif os.getenv("ORIGINAL_URL"):
            self.base_url = os.environ["ORIGINAL_URL"]

        if options.get("env"):
            self.env = get_environment(options["env"])
        elif os.getenv("ORIGINAL_ENV"):
            self.env = get_environment(os.environ["ORIGINAL_ENV"])

        if self.env == Environment.Development:
            self.base_url = DEVELOPMENT_BASE_URL
        elif self.env == Environment.Production:
            self.base_url = PRODUCTION_BASE_URL

        if options.get("api_version"):
            self.api_version = options["api_version"]
        elif os.getenv("API_VERSION"):
            self.api_version = os.environ["API_VERSION"]

        self.base_url = self.base_url.rstrip("/")

        self.token = jwt.encode(
            {"resource": "*", "action": "*", "user_id": "*", "api_key": api_key},
            api_secret,
        )

    def get_default_params(self) -> Dict[str, str]:
        return {"api_key": self.api_key, "api_secret": self.api_secret}

    def create_token(self) -> str:
        return jwt.encode(
            {"resource": "*", "action": "*", "user_id": "*", "api_key": self.api_key},
            self.api_secret,
        )

    def create_search_params(
        self,
        filter_conditions: Dict,
        query: Union[str, Dict],
        **options: Any,
    ) -> Dict[str, Any]:
        params = options.copy()
        if isinstance(query, str):
            params.update({"query": query})
        else:
            params.update({"message_filter_conditions": query})

        params.update({"filter_conditions": filter_conditions})

        return params

    def handle_parsed_response(
        self, parsed_result: dict, reason: str, status: int, headers: dict
    ) -> OriginalResponse:
        if is_error_status_code(status):
            parse_and_raise_error(parsed_result, reason, status)
        return OriginalResponse(parsed_result, headers, status)

    @abc.abstractmethod
    def create_user(
        self,
        email: Union[None, str] = None,
        client_id: Union[None, str] = None,
        user_external_id: Union[None, str] = None,
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original user.
        """
        pass

    @abc.abstractmethod
    def get_user(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original user.

        :param uid: the user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_user_by_email(
        self, email: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Gets a user by email

        :param email: the user's email
        :return:
        """
        pass

    @abc.abstractmethod
    def get_user_by_client_id(
        self, client_id: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Deprecated: use get_user_by_user_external_id instead.

        Gets a user by client_id

        :param client_id: the user's client_id
        :return:
        """
        pass

    @abc.abstractmethod
    def get_user_by_user_external_id(
        self, user_external_id: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Gets a user by user_external_id

        :param user_external_id: the user's user_external_id
        :return:
        """
        pass

    @abc.abstractmethod
    def get_collection(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original collection.

        :param uid: the collection uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_asset(
        self, **asset_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original asset.

        :param asset_data: the asset data
        :return:
        """
        pass

    @abc.abstractmethod
    def edit_asset(
        self, uid: str, **asset_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Update an Original asset.

        :param uid: the asset uid
        :param asset_data: the asset data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_asset(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original asset.

        :param uid: the asset uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_assets_by_user_uid(
        self, user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get a list of Original assets by user uid.

        :param user_uid: the user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_transfer(
        self, **transfer_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original transfer.

        :param transfer_data: the transfer data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_transfer(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original transfer.

        :param uid: the transfer uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_transfers_by_user_uid(
        self, user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get a list of Original transfers by user uid.

        :param user_uid: the user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_burn(
        self, **burn_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original burn.

        :param burn_data: the burn data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_burn(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original burn.

        :param uid: the burn uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_burns_by_user_uid(
        self, user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get a list of Original burns by user uid.

        :param user_uid: the app user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_deposit(
        self, user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original deposit by user uid.

        :param user_uid: the user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_reward(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original reward.

        :param uid: the reward uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_allocation(
        self, **allocation_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original allocation.

        :param allocation_data: the allocation data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_allocation(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original allocation.

        :param uid: the allocation uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_allocations_by_user_uid(
        self, user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get a list of Original allocations by user uid.

        :param user_uid: the app user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_claim(
        self, **claim_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original claim.

        :param claim_data: the claim data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_claim(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original claim.

        :param uid: the claim uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_claims_by_user_uid(
        self, user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get a list of Original claims by user uid.

        :param user_uid: the app user uid
        :return:
        """
        pass



import json
import warnings
from typing import Any, Callable, Dict, Tuple, Union

import requests

from original_sdk.types.exceptions import ClientError

from .__pkg__ import __version__
from .base.client import BaseOriginalClient
from .types.original_response import OriginalResponse
from .utils import get_default_error_message


def get_user_agent() -> str:
    return f"original_sdk-python-client-{__version__}"


def get_default_header() -> Dict[str, str]:
    base_headers = {
        "Content-type": "application/json",
        "X-ORIGINAL-Client": get_user_agent(),
    }
    return base_headers


class OriginalClient(BaseOriginalClient):
    def __init__(
        self, api_key: str, api_secret: str, timeout: float = 6.0, **options: Any
    ):
        super().__init__(
            api_key=api_key, api_secret=api_secret, timeout=timeout, **options
        )
        self.session = requests.Session()
        self.session.mount("http://", requests.adapters.HTTPAdapter(max_retries=1))
        self.session.mount("https://", requests.adapters.HTTPAdapter(max_retries=1))

    def set_http_session(self, session: requests.Session) -> None:
        """
        You can use your own `requests.Session` instance. This instance
        will be used for underlying HTTP requests.
        """
        self.session = session

    def _get_response_details(
        self, response: requests.Response
    ) -> Tuple[Any, Dict[str, str], int]:
        try:
            json_response = response.json()
            headers = dict(response.headers)
            status = response.status_code
            return json_response, headers, status
        except ValueError:
            message = get_default_error_message(response.status_code) or response.text
            raise ClientError(
                message=message, status=response.status_code, data=message
            )

    def _parse_response(self, response: requests.Response) -> OriginalResponse:
        parsed_result, headers, status = self._get_response_details(response)
        return self.handle_parsed_response(
            parsed_result, response.reason, response.status_code, headers
        )

    def _make_request(
        self,
        method: Callable[..., requests.Response],
        relative_url: str,
        params: Dict = None,
        data: Any = None,
    ) -> OriginalResponse:
        params = params or {}
        data = data or {}
        serialized = None
        default_params = self.get_default_params()
        default_params.update(params)
        headers = get_default_header()
        headers["Authorization"] = f"Bearer {self.token}"
        headers["X-API-KEY"] = self.api_key

        url = f"{self.base_url}/{self.api_version}/{relative_url}"
        if method.__name__ in ["post", "put", "patch"]:
            serialized = json.dumps(data)

        response = method(
            url,
            data=serialized,
            headers=headers,
            params=default_params,
            timeout=self.timeout,
        )
        return self._parse_response(response)

    def put(
        self, relative_url: str, params: Dict = None, data: Any = None
    ) -> OriginalResponse:
        return self._make_request(self.session.put, relative_url, params, data)

    def post(
        self, relative_url: str, params: Dict = None, data: Any = None
    ) -> OriginalResponse:
        return self._make_request(self.session.post, relative_url, params, data)

    def get(self, relative_url: str, params: Dict = None) -> OriginalResponse:
        return self._make_request(self.session.get, relative_url, params, None)

    def delete(self, relative_url: str, params: Dict = None) -> OriginalResponse:
        return self._make_request(self.session.delete, relative_url, params, None)

    def patch(
        self, relative_url: str, params: Dict = None, data: Any = None
    ) -> OriginalResponse:
        return self._make_request(self.session.patch, relative_url, params, data)

    def create_user(
        self,
        email: Union[None, str] = None,
        client_id: Union[None, str] = None,
        user_external_id: Union[None, str] = None,
    ) -> OriginalResponse:
        if client_id:
            warnings.warn(
                "client_id is deprecated, please use user_external_id instead",
                DeprecationWarning,
                stacklevel=2,
            )
        return self.post(
            "user",
            data={
                "email": email,
                "client_id": client_id,
                "user_external_id": user_external_id,
            },
        )

    def get_user(self, uid: str) -> OriginalResponse:
        return self.get(f"user/{uid}")

    def get_user_by_email(self, email: str) -> OriginalResponse:
        return self.get("user", params={"email": email})

    def get_user_by_client_id(self, client_id: str) -> OriginalResponse:
        warnings.warn(
            "get_user_by_client_id is deprecated, please use get_user_by_user_external_id instead",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.get("user", params={"client_id": client_id})

    def get_user_by_user_external_id(self, user_external_id: str) -> OriginalResponse:
        return self.get("user", params={"user_external_id": user_external_id})

    def get_collection(self, uid: str) -> OriginalResponse:
        return self.get(f"collection/{uid}")

    def create_asset(self, **asset_data: Any) -> OriginalResponse:
        return self.post("asset", data=asset_data)

    def edit_asset(self, uid: str, **asset_data: Any) -> OriginalResponse:
        return self.put(f"asset/{uid}", data=asset_data)

    def get_asset(self, uid: str) -> OriginalResponse:
        return self.get(f"asset/{uid}")

    def get_assets_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("asset", params={"user_uid": user_uid})

    def create_transfer(self, **transfer_data: Any) -> OriginalResponse:
        return self.post("transfer", data=transfer_data)

    def get_transfer(self, uid: str) -> OriginalResponse:
        return self.get(f"transfer/{uid}")

    def get_transfers_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("transfer", params={"user_uid": user_uid})

    def create_burn(self, **burn_data: Any) -> OriginalResponse:
        return self.post("burn", data=burn_data)

    def get_burn(self, uid: str) -> OriginalResponse:
        return self.get(f"burn/{uid}")

    def get_burns_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("burn", params={"user_uid": user_uid})

    def get_deposit(self, user_uid: str) -> OriginalResponse:
        return self.get("deposit", params={"user_uid": user_uid})

    def get_reward(self, uid: str) -> OriginalResponse:
        return self.get(f"reward/{uid}")

    def create_allocation(self, **allocation_data: Any) -> OriginalResponse:
        return self.post("reward/allocate", data=allocation_data)

    def get_allocation(self, uid: str) -> OriginalResponse:
        return self.get(f"reward/allocate/{uid}")

    def get_allocations_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("reward/allocate", params={"user_uid": user_uid})

    def create_claim(self, **claim_data: Any) -> OriginalResponse:
        return self.post("reward/claim", data=claim_data)

    def get_claim(self, uid: str) -> OriginalResponse:
        return self.get(f"reward/claim/{uid}")

    def get_claims_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("reward/claim", params={"user_uid": user_uid})



from enum import Enum
from typing import Union


class Environment(Enum):
    Development = "development"
    Production = "production"


def get_environment(env_input: Union[str, Environment]) -> Environment:
    if isinstance(env_input, Environment):
        return env_input
    try:
        return Environment(env_input.lower())
    except ValueError:
        raise ValueError(f"Invalid environment: {env_input}")



from enum import Enum
from typing import List, Optional, TypedDict, Union, cast


class ErrorDetail(TypedDict, total=False):
    message: str
    code: str
    field_name: Optional[str]


ErrorDetailList = List[ErrorDetail]


class Error(TypedDict):
    type: str
    detail: Union[ErrorDetail, ErrorDetailList]


class OriginalErrorData(TypedDict):
    success: bool
    error: Error


class OriginalErrorCode(Enum):
    client_error = "client_error"
    server_error = "server_error"
    validation_error = "validation_error"


class OriginalError(Exception):
    def __init__(
        self,
        message: str,
        status: int,
        data: Union[OriginalErrorData, str],
        code: OriginalErrorCode,
    ) -> None:
        super().__init__(message)
        self.message = message
        self.status = status
        self.data = data
        self.code = code.value

    def __str__(self) -> str:
        return f"{self.message} - {self.status} - {self.code} - {self.data}"


class ClientError(OriginalError):
    def __init__(
        self, message: str, status: int, data: Union[OriginalErrorData, str]
    ) -> None:
        super().__init__(message, status, data, OriginalErrorCode.client_error)


class ServerError(OriginalError):
    def __init__(
        self, message: str, status: int, data: Union[OriginalErrorData, str]
    ) -> None:
        super().__init__(message, status, data, OriginalErrorCode.server_error)


class ValidationError(OriginalError):
    def __init__(
        self, message: str, status: int, data: Union[OriginalErrorData, str]
    ) -> None:
        super().__init__(message, status, data, OriginalErrorCode.validation_error)


def is_error_status_code(status_code: int) -> bool:
    # Consider only client errors(400 - 499) and server errors(500 - 599) as errors.
    return status_code < 200 or status_code >= 400


def parse_and_raise_error(parsed_result: dict, reason: str, status: int) -> None:
    result: OriginalErrorData = cast(OriginalErrorData, parsed_result)
    error = result.get("error")
    if error:
        error_type = error.get("type")
        detail = error.get("detail")
        if isinstance(detail, list):
            detail = detail[0]

        message = detail.get("message") if detail else reason

        if error_type == OriginalErrorCode.server_error.value:
            raise ServerError(message=message, status=status, data=result)
        elif error_type == OriginalErrorCode.validation_error.value:
            raise ValidationError(message=message, status=status, data=result)
        else:
            raise ClientError(message=message, status=status, data=result)
    else:
        raise ClientError(
            message="No error found in response when one was expected",
            status=status,
            data=result,
        )


OriginalAPIException = OriginalError



import random
import string
from http import HTTPStatus


def get_random_string(length: int) -> str:
    characters = string.ascii_letters + string.digits
    random_string = "".join(random.choice(characters) for i in range(length))
    return random_string


def get_default_error_message(status_code: int) -> str:
    try:
        status = HTTPStatus(status_code)
        return status.phrase
    except ValueError:
        return "Unknown Error"



from .async_client import OriginalAsyncClient
from .client import OriginalClient
from .types.environment import Environment
from .types.exceptions import (
    ClientError,
    OriginalError,
    OriginalErrorCode,
    ServerError,
    ValidationError,
)

__all__ = [
    "OriginalClient",
    "OriginalAsyncClient",
    "Environment",
    "OriginalError",
    "ClientError",
    "ServerError",
    "ValidationError",
    "OriginalErrorCode",
]



from setuptools import find_packages, setup

install_requires = [
    "requests>=2.22.0,<3",
    "aiodns>=2.0.0",
    "aiohttp>=3.6.0,<4",
    "aiofile>=3.1,<4",
    "pyjwt>=2.0.0,<3",
    "typing_extensions; python_version < '3.8'",
]
tests_require = ["pytest<=8.1.1", "pytest-asyncio<=0.21.1", "python-dotenv"]
ci_require = [
    "black",
    "flake8",
    "flake8-isort",
    "flake8-bugbear",
    "pytest-cov",
    "mypy",
    "types-requests",
]

with open("README.md", "r") as f:
    long_description = f.read()

about = {}

with open("original_sdk/__pkg__.py") as fp:
    exec(fp.read(), about)

setup(
    name="original_sdk",
    version=about["__version__"],
    author=about["__maintainer__"],
    author_email=about["__email__"],
    url="https://github.com/GetOriginal/original-python",
    description="Python client for Original.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["*tests*"]),
    zip_safe=False,
    install_requires=install_requires,
    extras_require={"test": tests_require, "ci": ci_require},
    include_package_data=True,
    python_requires=">=3.7",
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Development Status :: 5 - Production/Stable",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
